<?php

/*	WikiMedia extension: LilyPond

To activate, edit your LocalSettings.php, add
	include('extensions/LilyPond.php');
and make sure that the images/ directory is writable.

Example wiki code: <lilypond>c d e f g</lilypond>

This is equivalent to the lilypond-book syntax
	<lilypond fragment relative=2>c d e f g</lilypond>
If you want something akin to lilypond-book's <lilybook>, use
	<lilybook>...</lilybook>
If you want to typeset a fragment with clickable midi, use
	<lilymidi>...</lilymidi>

You can set the global variable $wgLilypond if you want to override the
path to the Lilypond executable.

*/

$wgExtensionFunctions[] = "wfLilyPondExtension";

function wfLilyPondExtension() {
	global $wgParser;
	$wgParser->setHook( "lilypond", "renderLilyPondFragment" );
	$wgParser->setHook( "lilymidi", "renderLilyPondMidiFragment" );
	$wgParser->setHook( "lilybook", "renderLilyPond" );
}

if( !isset( $wgLilypond ) )
	$wgLilypond = "PATH=\$PATH:/usr/local/bin /usr/local/bin/lilypond";

function renderLilyPondMidiFragment( $lilypond_code )
{
	return renderLilyPondFragment( $lilypond_code, true );
}

function renderLilyPondFragment( $lilypond_code, $midi=false )
{
	return renderLilyPond("\\paper {\n"
			. "\traggedright = ##t\n"
			. "\tindent = 0\mm\n"
			. "}\n"
			. "\\score {\n"
			. "\t\\relative c'' {\n"
			. $lilypond_code
			. "\t}\n"
			. "\t\\layout { }\n"
			. ($midi?"\t\\midi { }\n":"")
			. "}\n", $lilypond_code );
}

function renderLilyPond( $lilypond_code, $short_code=false )
{
	global $wgMathPath, $wgMathDirectory, $wgTmpDirectory, $wgLilypond;

	$mf   = wfMsg( "math_failure" );
	$munk = wfMsg( "math_unknown_error" );

	$fname = "renderMusic";

	$md5 = md5($lilypond_code);

	if( file_exists( $wgMathDirectory."/".$md5.".midi" ) ) {
		$pre = "<a href=\"".$wgMathPath."/".$md5.".midi\">";
		$post = "</a>";
	} else {
		$pre = "";
		$post = "";
	}

	# if short_code is supplied, this is a fragment
	if( $short_code ) {
		$link = "<img src=\"".$wgMathPath."/".$md5.".png\" alt=\""
			.htmlspecialchars( $short_code )."\">";

		if( file_exists( "$wgMathDirectory/$md5.png" ) ) {
			return $pre.$link.$post;
		}
	} else {
		if(  file_exists( "$wgMathDirectory/$md5-1.png" ) ) {
			$link="";
			for($i=0; file_exists( $wgMathDirectory."/".$md5."-".$i.".png" ); $i++) {
				$link.="<img src=\"".$wgMathPath."/".$md5."-".$i.".png\" alt=\""
				.htmlspecialchars( ($i>1?"page ".$i:$lilypond_code) )."\">";
			}
			return $pre.$link.$post;
		}
	}
	
	# Ensure that the temp and output directories are available before continuing...
	if( !file_exists( $wgMathDirectory ) ) {
		if( !@mkdir( $wgMathDirectory ) ) {
			return "<b>$mf (" . wfMsg( "math_bad_output" ) . ")</b>";
		}
	} elseif( !is_dir( $wgMathDirectory ) || !is_writable( $wgMathDirectory ) ) {
		return "<b>$mf (" . wfMsg( "math_bad_output" ) . ")</b>";
	}
	if( !file_exists( $wgTmpDirectory ) ) {
		if( !@mkdir( $wgTmpDirectory ) ) {
			return "<b>$mf (" . wfMsg( "math_bad_tmpdir" ) . ")</b>";
		}
	} elseif( !is_dir( $wgTmpDirectory ) || !is_writable( $wgTmpDirectory ) ) {
		return "<b>$mf (" . wfMsg( "math_bad_tmpdir" ) . ")</b>";
	}

	$lyFile = $md5.".ly";
	$out = fopen( $wgTmpDirectory."/".$lyFile, "w" );
	if( $out === false ) {
		return "<b>$mf (" . wfMsg( "math_bad_tmpdir" ) . ")</b>";
	}
	fwrite( $out, $lilypond_code );
	fclose( $out );

	$cmd = $wgLilypond." --safe --format=ps --preview --png "
		. escapeshellarg($lyFile) . " 2>&1";

	wfDebug( "Lilypond: $cmd" );
	$oldcwd = getcwd();
	chdir( $wgTmpDirectory );
	$contents = exec( $cmd, $output, $ret );
	chdir( $oldcwd );

	if( $ret != 0 ) {
		return "<br><b>LilyPond error:</b><br><i>"
		. str_replace( array( $md5, " " ),
			array( "<b>your code</b>", "&nbsp;" ),
			nl2br( htmlentities( join( "\n", $output ) ) ) )
		. "</i><br>";
	}

	if($short_code) {
		$outputFile = $wgTmpDirectory."/".$md5.".preview.png";

		if( !file_exists( $outputFile ) ) {
			return "<b>$mf (" . wfMsg( "math_image_error" ) . ")</b>";
		}

		rename( $outputFile, $wgMathDirectory."/".$md5.".png");
	}

	# remove all temporary files
	$files = opendir( $wgTmpDirectory );
	$last_page = 0;
	while( false !== ($file = readdir( $files ))) {
		if( substr( $file, 0, 32 ) != $md5 )
			continue;

		$file_absolute = $wgTmpDirectory."/".$file;
		if( !$short_code && preg_match( '/-page(\d+)\.png$/', $file, $matches ) ) {
			if($matches[1]>$last_page)
				$last_page = $matches[1];
			rename( $file_absolute, $wgMathDirectory."/".$md5."-".$matches[1].".png" );
			continue;
		}

		if( preg_match( '/.midi$/', $file ) ) {
			rename( $file_absolute, $wgMathDirectory."/".$md5.".midi" );
			$pre = "<a href=\"".$wgMathPath."/".$md5.".midi\">";
			$post = "</a>";
			continue;
		}

		if( !is_file( $file_absolute ) )
			continue;
		unlink( $file_absolute );
	}
	closedir( $files );

	if( $short_code ) {
		if( !file_exists( $wgMathDirectory."/".$md5.".png" ) ) {
			$errmsg = wfMsg( "math_image_error" );
			return "<h3>$mf ($errmsg): " . htmlspecialchars($lilypond_code) . "</h3>";
		}
	} else {
		# delete blank page
		if($last_page>1)
			unlink( $wgMathDirectory."/".$md5."-".$last_page.".png" );
		else
			$last_page++;

		$link="";
		for($i=1; $i<$last_page; $i++)
			$link.="<img src=\"".$wgMathPath."/".$md5."-".$i.".png\" alt=\""
			.htmlspecialchars( "page ".$i )."\">";
	}

	return $pre.$link.$post;
}

?>
